@@ -21,15 +21,15 @@
 /**
  * SECTION:element-siddec
  *
- * This element decodes .sid files to raw audio. .sid files are in fact 
- * small Commodore 64 programs that are executed on an emulated 6502 CPU and a 
+ * This element decodes .sid files to raw audio. .sid files are in fact
+ * small Commodore 64 programs that are executed on an emulated 6502 CPU and a
  * MOS 6581 sound chip.
- * 
+ *
  * This plugin will first load the complete program into memory before starting
  * the emulator and producing output.
- * 
+ *
  * Seeking is not (and cannot be) implemented.
- * 
+ *
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
@@ -198,10 +198,8 @@ gst_siddec_class_init (GstSidDecClass * klass)
       "Codec/Decoder/Audio", "Use libsidplay to decode SID audio tunes",
       "Wim Taymans <wim.taymans@gmail.com>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_templ));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_templ));
+  gst_element_class_add_static_pad_template (gstelement_class, &src_templ);
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_templ);
 
   GST_DEBUG_CATEGORY_INIT (gst_siddec_debug, "siddec", 0,
       "C64 sid song player");
@@ -396,7 +394,7 @@ play_loop (GstPad * pad)
   GstSidDec *siddec;
   GstBuffer *out;
   GstMapInfo outmap;
-  gint64 value, offset, time;
+  gint64 value, offset, time = 0;
   GstFormat format;
 
   siddec = GST_SIDDEC (gst_pad_get_parent (pad));
@@ -410,29 +408,29 @@ play_loop (GstPad * pad)
 
   /* get offset in samples */
   format = GST_FORMAT_DEFAULT;
-  gst_siddec_src_convert (siddec->srcpad,
-      GST_FORMAT_BYTES, siddec->total_bytes, &format, &offset);
-  GST_BUFFER_OFFSET (out) = offset;
+  if (gst_siddec_src_convert (siddec->srcpad,
+        GST_FORMAT_BYTES, siddec->total_bytes, &format, &offset))
+    GST_BUFFER_OFFSET (out) = offset;
 
   /* get current timestamp */
   format = GST_FORMAT_TIME;
-  gst_siddec_src_convert (siddec->srcpad,
-      GST_FORMAT_BYTES, siddec->total_bytes, &format, &time);
-  GST_BUFFER_TIMESTAMP (out) = time;
+  if (gst_siddec_src_convert (siddec->srcpad,
+        GST_FORMAT_BYTES, siddec->total_bytes, &format, &time))
+    GST_BUFFER_TIMESTAMP (out) = time;
 
   /* update position and get new timestamp to calculate duration */
   siddec->total_bytes += siddec->blocksize;
 
   /* get offset in samples */
   format = GST_FORMAT_DEFAULT;
-  gst_siddec_src_convert (siddec->srcpad,
-      GST_FORMAT_BYTES, siddec->total_bytes, &format, &value);
-  GST_BUFFER_OFFSET_END (out) = value;
+  if (gst_siddec_src_convert (siddec->srcpad,
+        GST_FORMAT_BYTES, siddec->total_bytes, &format, &value))
+    GST_BUFFER_OFFSET_END (out) = value;
 
   format = GST_FORMAT_TIME;
-  gst_siddec_src_convert (siddec->srcpad,
-      GST_FORMAT_BYTES, siddec->total_bytes, &format, &value);
-  GST_BUFFER_DURATION (out) = value - time;
+  if (gst_siddec_src_convert (siddec->srcpad,
+        GST_FORMAT_BYTES, siddec->total_bytes, &format, &value))
+    GST_BUFFER_DURATION (out) = value - time;
 
   if ((ret = gst_pad_push (siddec->srcpad, out)) != GST_FLOW_OK)
     goto pause;
@@ -445,19 +443,16 @@ done:
   /* ERRORS */
 pause:
   {
-    const gchar *reason = gst_flow_get_name (ret);
-
     if (ret == GST_FLOW_EOS) {
       /* perform EOS logic, FIXME, segment seek? */
       gst_pad_push_event (pad, gst_event_new_eos ());
     } else if (ret < GST_FLOW_EOS || ret == GST_FLOW_NOT_LINKED) {
       /* for fatal errors we post an error message */
-      GST_ELEMENT_ERROR (siddec, STREAM, FAILED,
-          (NULL), ("streaming task paused, reason %s", reason));
+      GST_ELEMENT_FLOW_ERROR (siddec, ret);
       gst_pad_push_event (pad, gst_event_new_eos ());
     }
 
-    GST_INFO_OBJECT (siddec, "pausing task, reason: %s", reason);
+    GST_INFO_OBJECT (siddec, "pausing task, reason: %s", gst_flow_get_name (ret));
     gst_pad_pause_task (pad);
     goto done;
   }
@@ -776,6 +771,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    siddec,
+    sid,
     "Uses libsidplay to decode .sid files",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
