@@ -68,7 +68,8 @@ typedef enum {
   ASF_OBJ_DIGITAL_SIGNATURE_OBJECT,
   ASF_OBJ_SCRIPT_COMMAND,
   ASF_OBJ_MARKER,
-  ASF_OBJ_UNKNOWN_ENCRYPTION_OBJECT
+  ASF_OBJ_UNKNOWN_ENCRYPTION_OBJECT,
+  ASF_OBJ_METADATA_LIBRARY_OBJECT,
 } AsfObjectID;
 
 typedef enum {
