@@ -197,12 +197,12 @@ gst_rmdemux_base_init (GstRMDemuxClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rmdemux_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rmdemux_videosrc_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rmdemux_audiosrc_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rmdemux_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rmdemux_videosrc_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rmdemux_audiosrc_template);
   gst_element_class_set_static_metadata (element_class, "RealMedia Demuxer",
       "Codec/Demuxer",
       "Demultiplex a RealMedia file into audio and video streams",
@@ -678,6 +678,19 @@ gst_rmdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
   return res;
 }
 
+static void
+gst_rmdemux_free_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
+{
+  g_object_unref (stream->adapter);
+  gst_rmdemux_stream_clear_cached_subpackets (rmdemux, stream);
+  if (stream->pending_tags)
+    gst_tag_list_unref (stream->pending_tags);
+  if (stream->subpackets)
+    g_ptr_array_free (stream->subpackets, TRUE);
+  g_free (stream->index);
+  g_free (stream);
+}
+
 static void
 gst_rmdemux_reset (GstRMDemux * rmdemux)
 {
@@ -690,16 +703,9 @@ gst_rmdemux_reset (GstRMDemux * rmdemux)
   for (cur = rmdemux->streams; cur; cur = cur->next) {
     GstRMDemuxStream *stream = cur->data;
 
-    g_object_unref (stream->adapter);
-    gst_rmdemux_stream_clear_cached_subpackets (rmdemux, stream);
     gst_flow_combiner_remove_pad (rmdemux->flowcombiner, stream->pad);
     gst_element_remove_pad (GST_ELEMENT (rmdemux), stream->pad);
-    if (stream->pending_tags)
-      gst_tag_list_unref (stream->pending_tags);
-    if (stream->subpackets)
-      g_ptr_array_free (stream->subpackets, TRUE);
-    g_free (stream->index);
-    g_free (stream);
+    gst_rmdemux_free_stream (rmdemux, stream);
   }
   g_slist_free (rmdemux->streams);
   rmdemux->streams = NULL;
@@ -961,8 +967,7 @@ need_pause:
         gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
       }
     } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
-      GST_ELEMENT_ERROR (rmdemux, STREAM, FAILED,
-          (NULL), ("stream stopped, reason %s", reason));
+      GST_ELEMENT_FLOW_ERROR (rmdemux, ret);
       gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
     }
     return;
@@ -1439,7 +1444,8 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
         if (stream->flavor > 3) {
           GST_WARNING_OBJECT (rmdemux, "bad SIPR flavor %d, freeing it",
               stream->flavor);
-          g_free (stream);
+          g_object_unref (stream->pad);
+          gst_rmdemux_free_stream (rmdemux, stream);
           goto beach;
         }
 
@@ -1482,7 +1488,7 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
   } else {
     GST_WARNING_OBJECT (rmdemux, "not adding stream of type %d, freeing it",
         stream->subtype);
-    g_free (stream);
+    gst_rmdemux_free_stream (rmdemux, stream);
     goto beach;
   }
 
@@ -1922,10 +1928,20 @@ gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const guint8 * data, int length)
 
   tags = gst_rm_utils_read_tags (data, length, gst_rm_utils_read_string16);
 
-  GST_LOG_OBJECT (rmdemux, "tags: %" GST_PTR_FORMAT, tags);
+  if (tags) {
+    GstTagList *old_tags = rmdemux->pending_tags;
 
-  rmdemux->pending_tags =
-      gst_tag_list_merge (rmdemux->pending_tags, tags, GST_TAG_MERGE_APPEND);
+    GST_LOG_OBJECT (rmdemux, "tags: %" GST_PTR_FORMAT, tags);
+
+    rmdemux->pending_tags =
+        gst_tag_list_merge (old_tags, tags, GST_TAG_MERGE_APPEND);
+
+    gst_tag_list_unref (tags);
+    if (old_tags)
+      gst_tag_list_unref (old_tags);
+
+    gst_tag_list_set_scope (rmdemux->pending_tags, GST_TAG_SCOPE_GLOBAL);
+  }
 }
 
 static void
@@ -2562,7 +2578,8 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
     ret = GST_FLOW_OK;
   }
 
-  cret = gst_flow_combiner_update_flow (rmdemux->flowcombiner, ret);
+  cret = gst_flow_combiner_update_pad_flow (rmdemux->flowcombiner, stream->pad,
+      ret);
 
 beach:
   return cret;
